#ifndef _INTERPRETER_H_
#define _INTERPRETER_H_

#include <stdlib.h>

// Size of the CHIP-8/SCHIP font data.
#define INTERPRETER_FONT_DATA_LENGTH ((5+10)*16)
// Address of the lower bound of CHIP-8 memory.
#define INTERPRETER_MEMORY_MIN (0x200-INTERPRETER_FONT_DATA_LENGTH)
// Address of the upper bound of CHIP-8 memory.
#define INTERPRETER_MEMORY_MAX (0xE9F)
// Number of stack levels provided by the interpreter.
#define INTERPRETER_STACK_LEVELS (16)

// Flag used to denote that the interpreter is in SCHIP mode.
#define INTERPRETER_FLAGS_SCHIP             (0b00000001)
// Flag used to denote that sprites should wrap around the left and right edges of the screen.
#define INTERPRETER_FLAGS_SPRITES_WRAPPED_X (0b00000010)
// Flag used to denote that sprites should wrap around the top and bottom edges of the screen.
#define INTERPRETER_FLAGS_SPRITES_WRAPPED_Y (0b00000100)
// Flag used to denote that the keypad is flipped in the Y axis.
#define INTERPRETER_FLAGS_KEYPAD_FLIPPED_Y  (0b00001000)
// Flag used to denote that a detected key should block until it is released.
#define INTERPRETER_FLAGS_KEYPAD_BLOCKING   (0b00010000)
// Default interpreter flags.
#define INTERPRETER_FLAGS_DEFAULT         (INTERPRETER_FLAGS_SPRITES_WRAPPED_X|INTERPRETER_FLAGS_SPRITES_WRAPPED_Y)

// Helper macros used to decode CHIP-8 instructions.
#define INTERPRETER_REG_X interpreter_registers[(current_instruction >> 8) & 0x0F]
#define INTERPRETER_REG_Y interpreter_registers[(current_instruction >> 4) & 0x0F]
#define INTERPRETER_REG_F interpreter_registers[0x0F]
#define INTERPRETER_IMMEDIATE (current_instruction & 0xFF)

// General-purpose delay timer. It decrements 60 times per second until it reaches zero.
extern uint8_t interpreter_delay_timer;

// Delay timer that makes a sound when non-zero. It decrements 60 times per second until it reaches zero.
extern uint8_t interpreter_sound_timer;

extern uint8_t interpreter_flags;

// Resets the interpreter by unloading the program from memory and clearing registers to default values.
void interpreter_reset(void);

// Loads a program into the interpreter from ROM.
void interpreter_load_P(const uint8_t* program, size_t length);

// Fetches and executes a single CHIP-8 instruction.
void interpreter_execute_instruction(void);

// Run the CHIP-8 timers for one "tick". Should be called 60 times a second.
void interpreter_tick(void);

#endif
