#include <stdint.h>

#include <avr/io.h>
#include <util/delay.h>

#include "keypad.h"
#include "interpreter.h"

#define KEYPAD_PORT (PORTD)
#define KEYPAD_DDR  (DDRD)
#define KEYPAD_PIN  (PIND)

uint8_t map_key_code(uint8_t key) {
	switch (key & 0x0F) {	
		case 0x0: return 0xD;
		case 0x1: return 0x0;
		case 0x2: return 0x1;
		case 0x3: return 0x2;
		case 0x4: return 0x4;
		case 0x5: return 0x5;
		case 0x6: return 0x6;
		case 0x7: return 0x8;
		case 0x8: return 0x9;
		case 0x9: return 0xA;
		case 0xA: return 0xC;
		case 0xB: return 0xE;
		case 0xC: return 0x3;
		case 0xD: return 0x7;
		case 0xE: return 0xB;
		case 0xF: return 0xF;
	}
	return 0;
}

void keypad_init(void) {
	KEYPAD_DDR = 0x00;
	KEYPAD_PORT = 0xF0;
}

uint8_t keypad_get_key_state(uint8_t key, uint8_t flags) {
	key = map_key_code(key);
	uint8_t key_row = (key >> 2) & 3;
	if (flags & INTERPRETER_FLAGS_KEYPAD_FLIPPED_Y) key_row = 3 - key_row;
	KEYPAD_DDR = _BV(key & 3);
	_delay_us(100);
	return ~(KEYPAD_PIN >> 4) & _BV(key_row);
}