#ifndef _LCD_H_
#define _LCD_H_

#include <stdint.h>

// No special sprite effects are required.
#define LCD_SPRITE_FLAGS_NONE      (0b00000000)
// The sprite should be drawn at double size.
#define LCD_SPRITE_FLAGS_ZOOMED    (0b00000001)
// The sprite should wrap around the the left and right edges of the screen.
#define LCD_SPRITE_FLAGS_WRAPPED_X (0b00000010)
// The sprite should wrap around the the top and bottom edges of the screen.
#define LCD_SPRITE_FLAGS_WRAPPED_Y (0b00000100)

// Initialise the LCD.
void lcd_init(void);

// Switch the display on.
void lcd_display_on(void);

// Switch the display off.
void lcd_display_off(void);

// Clear the display.
void lcd_clear(void);

// Invert a pixel.
uint8_t lcd_invert_pixel(uint8_t x, uint8_t y);

// EOR a sprite
uint8_t lcd_eor_sprite(uint8_t* data, int8_t x, int8_t y, uint8_t width, uint8_t height, uint8_t flags);

// Scroll the LCD down 0..15 lines.
void lcd_scroll_down(uint8_t lines);

// Scroll the LCD up 0..15 lines.
void lcd_scroll_up(uint8_t lines);

// Scroll the LCD left four columns.
void lcd_scroll_left(void);

// Scroll the LCD right four columns.
void lcd_scroll_right(void);

#endif
