﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using System.Drawing;

namespace Vinegar.FontEncoder {
	class Program {

		class CharacterInformation {
			public byte Index { get; set; }
			public byte[] Data { get; set; }
		}

		static void Main(string[] args) {

			// Collect the characters into a list first (we need to sort them later).
			var characters = new List<CharacterInformation>(96);

			// Regex to extract the character index from the filename.
			var characterIndexRegex = new Regex(@"Font_(\d+)\.png");

			// Iterate over all PNG files.
			foreach (var characterFilename in Directory.GetFiles("../../../../font/characters", "Font_*.png")) {
				var m = characterIndexRegex.Match(characterFilename);
				if (m != null) {

					using (var b = new Bitmap(characterFilename)) {
						// Check the height.
						if (b.Height != 8) throw new InvalidDataException("Character " + characterFilename + " not 8 pixels high.");
						// Create an array to store the character data.
						var characterData = new byte[b.Width];
						// Get the character data from the Bitmap.
						for (int x = 0; x < characterData.Length; ++x) {
							for (int y = 0; y < 8; ++y) {
								characterData[x] <<= 1;
								if (b.GetPixel(x, y).GetBrightness() > 0.5f) characterData[x] |= 0x01;
							}
						}
						// Add the character to the list.
						characters.Add(new CharacterInformation() {
							Index = byte.Parse(m.Groups[1].Value),
							Data = characterData,
						});
					}
				}
			}

			// Sort.
			characters.Sort((a, b) => a.Index.CompareTo(b.Index));

			// Write the output file.
			using (var outputFile = new StreamWriter(File.Create(args[0]))) {

				// Start.
				outputFile.WriteLine("#ifndef _UI_FONT_H_");
				outputFile.WriteLine("#define _UI_FONT_H_");
				outputFile.WriteLine("#include <stdint.h>");
				outputFile.WriteLine("#include <avr/pgmspace.h>");
				outputFile.WriteLine("const uint8_t ui_font[] PROGMEM = {");

				// Output offsets.
				ushort offset = (ushort)(characters.Count * 2);
				foreach (var character in characters) {
					outputFile.WriteLine("\t(uint8_t){0},  (uint8_t)({0} >> 8), /* {1} */", offset, character.Index);
					offset += (ushort)(1 + character.Data.Length);
				}

				// Output data.
				foreach (var character in characters) {
					outputFile.Write("\t{0}, ", character.Data.Length);
					foreach (var dataByte in character.Data) outputFile.Write("0x{0:X2}, ", dataByte);
					outputFile.WriteLine("/* {0} */", character.Index);
				}

				// Finish.
				outputFile.WriteLine("}}; /* Total size: {0} bytes */", offset);
				outputFile.WriteLine("#endif");
			}
		}
	}
}
